@@ -3,6 +3,7 @@ import math
 import numpy as np
 import pandas as pd
 import logging
+import os
 
 from queue import Empty
 
@@ -17,17 +18,15 @@ from numerous.engine.system.external_mappings import ExternalMappingElement
 from numerous.utils.data_loader import DataLoader, DataFrame
 from numerous.utils.logger_levels import LoggerLevel
 from numerous.utils.historian import Historian
-from numerous.image_tools.job import NumerousSimulationJob
+from numerous.image_tools.job import NumerousSimulationJob, ExitCode
 from numerous.image_tools.system import NumerousSystem
 
-logger = logging.getLogger(__file__)
-
-
 try:
     FEPS = np.finfo(1.0).eps
 except AttributeError:
     FEPS = 2.220446049250313e-16
 
+LOG_LEVEL = os.getenv('LOG_LEVEL', logging.DEBUG)
 
 def map_to_dataframe(system: NumerousSystem, t: float, index_to_timestep_mapping: str):
     df_dict = {}
@@ -54,6 +53,8 @@ class NumerousEngineJob(NumerousSimulationJob):
         self.t_offset = None
         self.y0 = None
         self._metadata_written = False
+        self.logger = logging.getLogger("numerous.engine_tools.job")
+        self.logger.setLevel(level=LOG_LEVEL)
 
         self.dataframe_aliases = {}
         self.external_mappings = []
@@ -90,6 +91,7 @@ class NumerousEngineJob(NumerousSimulationJob):
                 self._queue = queue.Queue()
 
             def store(self, df):
+                logging.info("storing historian")
                 self.add_dataframe(df)
                 self.df = df
 
@@ -98,6 +100,8 @@ class NumerousEngineJob(NumerousSimulationJob):
                 output_dict.update({"_index": df['time'].values})  # best practise is to save relative time
 
                 outputs = [dict(zip(output_dict, t)) for t in zip(*output_dict.values())]
+                if not outputs:
+                    logging.info("no outputs in historian")
                 for output in outputs:
                     self._queue.put(output)
 
@@ -139,6 +143,7 @@ class NumerousEngineJob(NumerousSimulationJob):
     def initialize_simulation_system(self):
 
         self.dt= self.system.dt
+        self.logger.info(f"timestep is {self.dt} seconds")
         self.t_offset = self.system.start_time
 
         self.y0 = self.system.states
@@ -178,7 +183,6 @@ class NumerousEngineJob(NumerousSimulationJob):
         )
 
         # Create simulation object
-        #simulation = None
         simulation = Simulation(
             model, t_start=self.t_start, t_stop=self.t_stop,
             num=1, num_inner=1,
@@ -204,7 +208,7 @@ class NumerousEngineJob(NumerousSimulationJob):
         """
         return
 
-    def write_output(self, t, output):
+    def write_output(self, t, _):
         output = self.historian.get()
         if not output:
             return
@@ -212,6 +216,8 @@ class NumerousEngineJob(NumerousSimulationJob):
         if not self._metadata_written:
             self.app.client.set_timeseries_meta_data([{"name": tag} for tag in output.keys()],
                                                      offset=self.app.start_time)
+            self.logger.info("metadata written")
+            self._metadata_written = True
 
         while output:
             self.app.output.write_row(t, output)
@@ -227,16 +233,15 @@ class NumerousEngineJob(NumerousSimulationJob):
     def step(self, t: float = None, dt: float = None, init=False):
 
         t_ = self.simulation.solver.info.t if self.simulation.solver.info else 0
-        #self._update_input(t_)
-        # watch out for round-off error!
         _, t_step = self.simulation.step_solve(t_, min(dt, self.t_simulation-t_))
 
-        if t_step >= self.t_simulation:
-            self.simulation.model.create_historian_df()
-
         return t_step + self.t_offset, None
 
-
+    def post_run(self, exit_code: ExitCode):
+        if exit_code.value > 0:
+            self.logger.info("creating final historian")
+            self.simulation.model.create_historian_df()
+            self.write_output(self._t, None)
 
 
 
